@@ -2,6 +2,7 @@ import time
 import requests
 import base64, hashlib, hmac
 import json
+import math
 import calendar
 import warnings
 import logging
@@ -9,6 +10,7 @@ import functools
 import numpy as np
 import pandas as pd
 import datetime as dt
+import timedelta as td
 from collections import namedtuple
 from kucoincli.utils._utils import _parse_date
 from kucoincli.utils._utils import _parse_interval
@@ -303,26 +305,29 @@ class Client(BaseClient):
             raise KucoinResponseError("No sub-accounts found")
         return resp
 
-    def recent_orders(self, page:int=1, pagesize:int=50, id:str=None) -> pd.DataFrame:
+    def recent_orders(self, id:str=None, unix:bool=False, page:int=1, pagesize:int=500) -> pd.DataFrame:
         """Returns DataFrame with details of all trades placed in last 24 hours.
 
         Notes
         -----
-            - Max trades per page is 500; Min trades per page is 10
-            - Max number of trades returned (across all pages) is 1000
-            - Data is paganated into n pages displaying `pagesize` number of trades
+        - Max trades per page is 500; Min trades per page is 10
+        - Max number of trades returned (across all pages) is 1000
+        - Data is paganated into n pages displaying `pagesize` number of trades
 
         Parameters
         ----------
+        id : str, optional
+            Specify an explicit order ID to return values for only that order
+        unix : bool, optional
+            If `unix=False` [default], return datetime values as datetime objects.
+            If `unix=True`, return datetime as unix epochs at millisecond precision.
         page : int 
             (Optional) JSON response is paganated. Use this variable
             to control the page number viewed. Default value returns first 
             page of paganated data.
         pagesize : int 
             (Optional) Max number of trades to display per response
-            Default `pagesize` is 50.
-        id : str, optional
-            Specify an explicit order ID to return values for only that order
+            Default `pagesize` is 500.
         
         Returns
         -------
@@ -339,32 +344,40 @@ class Client(BaseClient):
             raise KucoinResponseError("No orders in the last 24 hours or order ID not found.")
         if not id:
             df = pd.DataFrame(resp).squeeze()
-            df["createdAt"] = pd.to_datetime(df["createdAt"], unit="ms")
-            df = df.set_index("createdAt")
+            if not unix:
+                df["createdAt"] = pd.to_datetime(df["createdAt"], unit="ms")
+                df = df.set_index("createdAt")
         else:
             df = pd.Series(resp)
-            df.createdAt = pd.to_datetime(df.createdAt, unit="ms")
+            if not unix:
+                df.createdAt = pd.to_datetime(df.createdAt, unit="ms")
         return df
 
     def transfer(
-        self, currency:str, source_acc:str, dest_acc:str, amount:float, oid:str=None
+        self, currency:str, source_acc:str, dest_acc:str, amount:float, oid:str=None,
+        from_pair:str=None, to_pair:str=None,
     ) -> dict:
-        """Function for transferring funds between margin, trade and main accounts.
+        """Function for transferring funds between cross, isolated, trade, and main accounts.
 
         Parameters
         ----------
-        currency : str 
-            Currency to transfer between accounts (e.g., BTC-USDT).
-        source_acc : str 
-            Source account type. Options are: Main, trade, and margin.
+        currency : str
+            Currency to transfer between accounts (e.g., BTC).
+        source_acc : str
+            Source account type. Current options: `['main', 'trade', 'margin', 'isolated']`
         dest_acc : str 
-            Destination account type. Options are: Main, trade, and margin.
-        amount : float 
-            Positive float value. Must be of the transfer currency precision.
-        oid : str
-            (Optional) Unique order ID for identification of transfer. OID will 
-            autogenerate an integer based on the UNIX epoch if not explicitly 
-            given.
+            Destination account type. Options are: `['main', 'trade', 'margin', 'isolated']`
+        amount : float
+            Currency amount to transfer. Must be of the transfer currency precision.
+        oid : str, optional
+            Unique order ID for identification of transfer. OID will autogenerate
+            if not provided.
+        from_pair : str, optional
+            Specify trading pair (e.g., BTC-USDT) to transfer assets from. Only required 
+            when `source_acc='isolated'`.
+        to_pair : str, optional
+            Specify trading pair (e.g., BTC-USDT) to transfer assets to. Only required
+            when `dest_acc='isolated'`.
         
         Returns
         -------
@@ -382,6 +395,14 @@ class Client(BaseClient):
             data["clientOid"] = oid
         else:
             data["clientOid"] = str(int(time.time() * 10000))
+        if source_acc == 'isolated':
+            if not from_pair:
+                raise ValueError('Must specify `from_pair` when transfering from isolated')
+            data['fromTag'] = from_pair.upper()
+        if dest_acc == 'isolated':
+            if not to_pair:
+                raise ValueError('Must specify `to_pair` when transfering to isolated')
+            data['toTag'] = to_pair.upper()
         resp = self._request(
             "post", path, signed=True, api_version=self.API_VERSION2, data=data
         )
@@ -541,23 +562,23 @@ class Client(BaseClient):
         
         Parameters
         ----------
-        currency : str or list or None, optional
+        currency : str or list, optional
             Specify a single pair or list of pairs to query. If `currency=None`, return 
             all trading pairs
-        market : str or list or None, optional
+        market : str or list, optional
             Filter response by trading market. If `market=None` return all markets. Markets:
             `['ALTS', 'BTC', 'DeFi', 'NFT', 'USDS', 'KCS', 'Polkadot', 'ETF', 'FIAT']`
-        base : str or list or None, optional
+        base : str or list, optional
             Specify explicit base currency or list of currencies. If `base_curr=None` 
             [DEFAULT], all base currencies will be returned.
-        quote : str or list or None, optional
-            Specify explicit quote currency or list of currencies. If `base_curr=None` 
+        quote : str or list, optional
+            Specify explicit quote currency or list of quote currencies. If `quote_curr=None` 
             [DEFAULT], all quote currencies will be returned.
-        marginable : bool or None
+        marginable : bool, optional
             If `marginable=True`, return only marginable securities. If `marginable=False` return only
             securities which cannot be traded on margin. If `marginable=None` [DEFAULT] return all
             trading pairs regardless of marginability.
-        tradable : bool or None
+        tradable : bool, optional
             If `tradable=True`, return trading-enabled securities. If `trading=False` return untradable
             listed pairs. If `tradable=None` [DEFAULT], return all securites regardless of tradability.
 
@@ -582,8 +603,10 @@ class Client(BaseClient):
         df = pd.DataFrame(resp["data"]).set_index("name")
         if pair:
             try:
+                pair = [pair] if isinstance(pair, str) else pair
+                pair = [symbol.upper() for symbol in pair]
                 df = df.loc[pair, :]            
-            except KeyError as e: 
+            except KeyError as e:
                 raise KeyError("Keys not found in response data", e)
         if marginable is not None:
             df = df[df["isMarginEnabled"] == marginable]
@@ -626,31 +649,32 @@ class Client(BaseClient):
         df.set_index("timestamp", inplace=True)
         return df
 
-    def lending_rate(self, currency:str, term:int=None) -> pd.DataFrame:
+    def lending_rate(self, currency:str, term:int=None, maxrate:float=None) -> pd.DataFrame:
         """Query API to obtain current a list of available margin terms
 
-        Notes
-        -----
-            Sorted descending on sequence of daily interest rate and term
-
         Parameters
         ----------
         currency : str
             Target currency to pull lending rates on (e.g., BTC)
+        term : int, optional
+            Specify term details 
         
         Returns
         -------
         DataFrame
-            Returns pandas DataFrame containing margin rate details
+            Returns pandas DataFrame containing margin rate details.
         """
         path = f"margin/market?currency={currency.upper()}"
         if term:
             path = path + f"&term={term}"
         resp = self._request("get", path)
-        if df.empty:
-            KucoinResponseError("No results for currency and term combination")
         df = pd.DataFrame(resp["data"])
-        return df
+        if df.empty:
+            err_msg = f"No results for {currency} returned"
+            if term:
+                err_msg += f" @ {term} day term"
+            raise KucoinResponseError(err_msg)
+        return df.astype(float)
 
     def get_trade_history(self, pair:str, ascending:bool=False) -> pd.DataFrame:
         """Query API for most recent 100 filled trades for target pair
@@ -690,8 +714,7 @@ class Client(BaseClient):
         return resp["data"]
 
     def margin_account(
-        self, asset:None or str or list=None, balance:None or float=None, 
-        mode:str="cross",
+        self, asset:str or list=None, balance:float=None, mode:str="cross",
     ) -> pd.DataFrame or pd.Series:
         """Return cross margin account details
 
@@ -1210,7 +1233,7 @@ class Client(BaseClient):
             data["funds"] = funds
         if margin:
             path = "margin/order" # Update to margin path
-            data["marginMode"] = mode
+            data["marginModel"] = mode
             data["autoBorrow"] = autoborrow
         if type == "limit":
             data["price"] = price
@@ -1248,15 +1271,15 @@ class Client(BaseClient):
             Specific currency to repay liabilities against (e.g., BTC).
         size : float, optional
             Total sum to repay (in currency terms). Must be a multiple of currency max
-            precision. If `size=None` [defualt], repayment size is max of total margin
+            precision. If `size=None` [default], repayment size is max of total margin
             debt or total available balance.
         id : str or None, optional
             Repay by specific order ID. If `id=None` [default], repayment will occur
             across all borrowings for `currency` in `priority` order.
         priority : str, optional
             Specify how to prioritize debt repayment.
-            - Highest: [default] Repay highest interest rate loans first
-            - Soonest: Repay nearest term loans first 
+            - `highest`: [default] Repay highest interest rate loans first
+            - `soonest`: Repay loans with shortest remaining term first 
 
         Returns
         -------
@@ -1339,7 +1362,7 @@ class Client(BaseClient):
         return res.squeeze()
 
     def margin_balance(
-        self, currency:str or list=None, unix:bool=False, page:int=None
+        self, currency:str or list=None, unix:bool=False, id:str or list=None, page:int=None
     ) -> pd.DataFrame:
         """Query detailed information regarding current margin balances against the user's account.
         
@@ -1357,9 +1380,9 @@ class Client(BaseClient):
         
         Returns
         -------
-        DataFrame of Series
-            Return pandas DataFrame with list outstanding margin debts. If only a single debt is 
-            outstanding returns a series. If no debts are outstanding, returns an empty dataframe.
+        DataFrame
+            Return pandas DataFrame with list outstanding margin debts. If no debts are 
+            outstanding, returns an empty dataframe.
         """
         concat_paginated = False
         if not page:
@@ -1385,7 +1408,10 @@ class Client(BaseClient):
         if not unix:
             res['createdAt'] = pd.to_datetime(res['createdAt'], unit='ms')
             res['maturityTime'] = pd.to_datetime(res['maturityTime'], unit='ms')
-        return res.squeeze()
+        if id:
+            id = [id] if isinstance(id, str) else id
+            res = res[res['tradeId'].isin(id)]
+        return res
     
     def get_outstanding_loans(
         self, currency:str or list=None, page:int=None, pagesize:int=50
@@ -1532,17 +1558,40 @@ class Client(BaseClient):
         resp = self._request("post", path, data=data, signed=True)
         return resp
 
-    def get_borrow_order(self, id):
-        """Get borrow order details for specific order ID"""
-        path = f"margin/borrow?orderId={id}"
-        resp = self._request("get", path, signed=True)
-        return resp
-
     def borrow(
         self, currency:str, size:float, maxrate:float=None, 
-        type:str="FOK", term:int or list or None=None,
+        type:str="IOC", term:int or list or None=None,
     ) -> dict:
-        """Post borrow request to KuCoin lending markets"""
+        """Post borrow request to KuCoin lending markets
+        
+        Parameters
+        ----------
+        currency : str
+            Specify currency to borrow
+        size : float
+            Total borrow size
+        maxrate : float, optional
+            Maximum acceptable daily interest rate. If not specified, all rates will
+            be accepted.
+        term : int or list, optional
+            Specify acceptable term length or list of term lengths. If not specified,
+            all term lengths will be accepted. Currently supportered term lengths:
+            `[7, 14, 28]`. Term lengths are measured in days
+        type : str, optional
+            Execution type. Borrow orders supports `FOK` or `IOC` orders.
+            * `IOC`: Immediate or Cancel. This is the detault order submission. 
+              Maximum portion of borrow order that is immediately fillable will 
+              be executed with the remaining portion immediately cancelled.
+            * `FOK`: Fill or Kill. FOK orders will either be filled in their
+              entirety or immediately cancelled. No partial fulfillment is
+              allowed.
+
+        Returns
+        -------
+        dict
+            Returns dictionary with order details. Use orderId with 
+            `.get_borrow_order` to obtain order details.
+        """
         data = {"currency": currency, "type": type, "size": size}
         if maxrate:
             data["maxRate"] = maxrate
@@ -1554,19 +1603,46 @@ class Client(BaseClient):
         resp = self._request("post", path, data=data, signed=True)
         return resp
 
-    def cancel_order(self, symbols=None, type='spot', id=None, oid=None):
-        """Cancel all orders or specific order (via OID)
-        
-        symbol : str or list, optional
-        type : str
-            `['trade', 'cross', 'isolated']`
-        id : str or list
-            Unique order ID
-        
+    def cancel_order(
+        self, symbols:str or list=None, type:str='trade', oid:str or list=None, 
+        cid:str or list=None,
+    ) -> list:
+        """Cancel orders by ID or cancel all orders related to a specific currency pair
+
+        Use this endpoint to cancel single orders or to submit batches of cancellations.
+        You are able to submit blended batches with some CID/OID lists alongside lists of 
+        symbols (although all orders must be in the same account type). It is highly recommended
+        that you avoid overlap in your cancellation batches. For example, users should avoid 
+        submitting an order ID corresponding to an ETH-USDT order while also submitting the 
+        cancellation of all ETH-USDT orders. Be aware that each ID and symbol pair added for
+        cancellation requires additional API calls on the backend. For best performance, 
+        work to consolidate the number of total requests when submitting cancellations.
         
+        Parameters
+        ----------
+        symbols : str or list, optional
+            Specify a currency pair or list of currency pairs (e.g. 'ETH-USDT') for which to 
+            cancel ALL outstanding orders. Users must specify one or more of `symbols`, `id`,
+            or `oid` arguments.
+        type : str, optional
+            Specify market in which to submit cancellations. Current markets are 
+            `['trade', 'cross', 'isolated']`. Default is the trade (spot) market.
+        oid : str or list, optional
+            Specify one or more orders by OID. OID are assigned by KuCoin upon order
+            submission. Users must specify one or more of `symbols`, `id`,
+            or `oid` arguments.
+        cid : str or list, optional
+            Specify one or more orders by unique client-assigned ID. Client IDs are attached
+            to orders by the client upon order submission. Users must specify one or more 
+            of `symbols`, `id`, or `oid` arguments.
+
+        Response
+        --------
+        list
+            List of cancelled order IDs
         """
         if not symbols and not id and not oid:
-            raise ValueError("Must specify one of `symbols`, `id` or `oid`")
+            raise ValueError("Must specify one of `symbols`, `id`, or `oid`")
 
         if type == 'cross' or type == 'margin':
             type = 'MARGIN_TRADE'
@@ -1577,21 +1653,21 @@ class Client(BaseClient):
 
         cancellations = []
 
-        if id:
-            ids = [id] if isinstance(id, str) else id
-            for id in ids:
-                path = f"orders/{oid}"
+        if cid:
+            cids = [cid] if isinstance(cid, str) else cid
+            for cid in cids:
+                path = f"order/client-order/{cid}"
                 cancellations.append(path)
+        if oid:
+            oids = [oid] if isinstance(oid, str) else oid
+            for oid in oids:
+                path = f"orders/{oid}"
+                cancellations.append(oid)
         if symbols:
             symbols = [symbols] if isinstance(symbols, str) else symbols
             for symbol in symbols:
                 path = f"orders?symbol={symbol}&tradeType={type}"
                 cancellations.append(path)
-        if oid:
-            oids = [oid] if isinstance(oid, str) else oid
-            for oid in oids:
-                path = f"order/client-order/{oid}"
-                cancellations.append(oid)
 
         responses = []
         for path in cancellations:
@@ -1603,74 +1679,178 @@ class Client(BaseClient):
                     "Order cancellation failure:" +
                     resp["msg"]
                 )
+                pass
 
         return responses
 
-    def list_orders(
-        self, status="done", symbols=None, side=None, type="trade",
-        start=None, stop=None, page=None, id=None, oid=None,
-    ):
-        """Get list of active and done orders"""
-        if type == 'cross' or type == 'margin':
-            type = 'MARGIN_TRADE'
-        elif type == 'isolated':
-            type = 'MARGIN_ISOLATED_TRADE'
+    def order_history(
+        self, acc_type:str="trade", symbols:str or list=None, start:str=None, end:str=None,
+        side:str=None, status:str="done", order_type:str or list=None, consolidated:bool=True,
+        page:int=None, id:str or list=None, oid:str or list=None, channel:str or list=None,
+    ) -> pd.DataFrame:
+        """Detailed cross account information on both active and completed orders
+
+        This function returns order details in two flavors: Consolidated and unconsolidated. 
+        The default consolidated response contains only order price, order size, value of
+        the order portion that was filled, amount of order size that was filled, fees paid, 
+        and relevant asset details. The alternative unconsolidated response contains an 
+        additional 20+ detail columns that will, for most users, not be useful. Use the 
+        `consolidated` argument to toggle between these responses.
+
+        Parameters
+        ----------
+        acc_type : str, optional
+            Specify which account type to return active/completed trades from. Valid account 
+            types are `['trade', 'cross', 'isolated']`. Default account type is trade.
+        symbols : str or list, optional
+            Return only specified symbol or list of symbols (e.g., `['BTC-USDT', 'ETH-USDT']`)
+        start : dt.datetime or str, optional
+            Specify earliest date to obtain trade history from. String dates should be in 
+            YYYY-MM-DD format with HH:MM:SS additioionally accepted (e.g., '2022-01-01 00:00:00').
+            If `start` is not specified, default behavior will be to return last 7 days of order
+            history.
+        end : dt.datetime or str, optional
+            Specify latest date to obtain trade history from. String formatting requirements are
+            identical to `start`. If `end` is specified, but not `start`, a ValueError will be
+            raised.
+        side : str, optional
+            Return only `'buy'` or `'sell'` orders. By default both buy and sell with be returned.
+        status : str, optional
+            Return only active or only completed orders. For completed orders use `status='done'` 
+            [default]. For active use, `status='active'`.
+        order_type : str or list, optional
+            Filter by order type(s). All order types are returned by default. Valid order types: 
+            `['limit', 'market', 'limit_stop', 'market_stop']`.
+        consolidated : bool, optional
+            If `True` [default], response is limited to most relevant columns. Set to `False`, 
+            for full response data.
+        page : int, optional
+            For better execution performance, specify a target page. If `page=None` [default],
+            all pages will be concatenated into one response. This process does require multiple
+            API calls assuming there is more than one page of responses and as such preformance will
+            be reduced.
+        id : str or list, optional
+            Filter response by ID or list of IDs.
+        oid : str or list, optional
+            Filter response by client OID (client OID are user assigned IDs attached to the order at
+            time of submission).
+        channel : str, optional
+            Filter by order entry channel. Valid channels include `['API', 'ANDROID', 'IOS', 'WEB']`
+
+        Returns
+        -------
+        DataFrame
+            Returns pandas DataFrame with historic order details. If no orders data is 
+            found, an empty dataframe will be returned.
+
+        See Also
+        --------
+        * `.recent_orders`: Light-weight function for obtaining orders placed in the last 24-hours
+        
+        Notes
+        -----
+        * Be aware that extended date ranges and large trade counts within a date ranges will
+        significantly impact function execution time. For quicker execution time, limit date ranges
+        as possible.
+        * Per 7 day interval, KuCoin will only return a maximum of 50,000 trades. If you are
+        executing / canceling more than 50,000 trades per 7 day period, this is not an appropriate
+        endpoint.
+        * Record of cancelled trades will only be maintained for 30 days by KuCoin's servers.
+        """
+        if end and not start:
+            raise ValueError('Connect specify `end` without also specifying `start`')
+        if acc_type == 'cross' or acc_type == 'margin':
+            acc_type = 'MARGIN_TRADE'
+        elif acc_type == 'isolated':
+            acc_type = 'MARGIN_ISOLATED_TRADE'
         else:
-            type = 'TRADE'
+            acc_type = 'TRADE'
 
         if start:
             if isinstance(start, str):
-                start = _parse_date(start, as_unix=True)
-            if isinstance(start, dt.datetime):
-                start = int(time.mktime(start.timetuple()))
+                start = _parse_date(start, as_unix=False)
+        if end:
+            if isinstance(end, str):
+                end = _parse_date(end, as_unix=False)
+        else:
+            end = dt.datetime.now()
 
-        if stop:
-            if isinstance(stop, str):
-                stop = _parse_date(stop, as_unix=True)
-            if isinstance(stop, dt.datetime):
-                stop = int(time.mktime(stop.timetuple()))
-
-        concat_paginated = False
-        if not page:
-            concat_paginated = True
-            page = 1
-
-        path = f"orders?status={status}&tradeType={type}&currentPage={page}"
         if start:
-            path += f"&startAt={start*1000}"
-        if stop:
-            path += f"&stopAt={stop*1000}"
-        resp = self._request("get", path, signed=True)
+            intervals = math.ceil(td.Timedelta(end - start).days/7)
+        else:
+            intervals = 1
+        responses = [] # Container for dataframe responses over multiple intervals
 
-        dfs = []
-        df = pd.DataFrame(resp["data"]["items"])
-        dfs.append(df)
-        if concat_paginated == True:
-            diff = resp["data"]["totalPage"] - resp["data"]["currentPage"]
-            for page in range(2, diff+2):
-                path = f"orders?status={status}&tradeType={type}&currentPage={page}"
-                if start:
-                    path += f"&startAt={start*1000}"
-                if stop:
-                    path += f"&stopAt={stop*1000}"
-                resp = self._request("get", path, signed=True)
-                dfs.append(pd.DataFrame(resp["data"]["items"]))
-        res = pd.concat(dfs).squeeze()
+        for _ in range(intervals):
+            concat_paginated = False
+            if not page:
+                concat_paginated = True
+                page = 1
+            path = f"orders?status={status}&tradeType={acc_type}&currentPage={page}&pageSize=500"
+            if start:
+                unix_start = int(time.mktime(start.timetuple()))
+                path += f"&startAt={unix_start*1000}"
+            resp = self._request("get", path, signed=True)
+
+            dfs = []
+            df = pd.DataFrame(resp["data"]["items"])
+            dfs.append(df)
+            if concat_paginated == True:
+                diff = resp["data"]["totalPage"] - resp["data"]["currentPage"]
+                # Rotate through additional results pages when neccesary
+                for page in range(2, diff+2):
+                    path = f"orders?status={status}&tradeType={acc_type}&currentPage={page}&pageSize=500"
+                    if start:
+                        path += f"&startAt={unix_start*1000}"
+                    resp = self._request("get", path, signed=True)
+                    temp_df = pd.DataFrame(resp["data"]["items"])
+                    dfs.append(temp_df)
+            res = pd.concat(dfs)
+            responses.append(res)
+            page = None # Reset page to none, this will force repeat page iteration
+            if start:
+                start += dt.timedelta(days=7)
         
+        res = pd.concat(responses, axis=0)
+        if res.empty:
+            return res
+        # Add a unix bool flag here
         res['createdAt'] = pd.to_datetime(res['createdAt'], unit='ms')
         if isinstance(res, pd.DataFrame):
             res.set_index('createdAt', inplace=True)
+            res.sort_values('createdAt', inplace=True)
+            end += dt.timedelta(days=1)
+            res = res.loc[:end]
 
-        if symbols:
-            symbols = [symbols] if isinstance(symbols, str) else symbols
-            res = res[res['symbol'].isin(symbols)]
         if id:
             id = [id] if isinstance(id, str) else id
             res = res[res['id'].isin(id)]
         if oid:
             oid = [oid] if isinstance(oid, str) else oid
             res = res[res['clientOid'].isin(oid)]
+        if channel:
+            res = res[res['channel'] == channel]
+        if order_type:
+            order_type = [order_type] if isinstance(order_type, str) else order_type
+            res = res[res['type'].isin(order_type)]
+        if symbols:
+            symbols = [symbols] if isinstance(symbols, str) else symbols
+            res = res[res['symbol'].isin(symbols)]
         if side:
-            res = res[res['side'] == side]            
+            res = res[res['side'] == side]
+        if consolidated:
+            consol_columns = [
+                'symbol', 'type', 'side', 'price', 'size', 
+                'dealFunds', 'dealSize', 'fee'
+            ]
+            res = res[consol_columns]
+        float_cols = ['price', 'size', 'dealFunds', 'dealSize','fee']
+        res[float_cols] = res[float_cols].astype(float)
+        # somehow duplicates are getting into the response...
+        return res.drop_duplicates().sort_index(ascending=False)
 
-        return res
+    def get_borrow_order(self, id):
+        """Use orderId from `.borrow` to obtain loan details"""
+        path = f"margin/borrow?orderId={id}"
+        resp = self._request("get", path, signed=True)
+        return resp
